//   Array.h
#pragma once
#include <stdio.h>
#include <new>

#define _DEBUG_IND_CHECK

// declarations

template <class T>
class Array
{
 private:
    T * value;
    int size;
    int maxsize;
 public:
    Array(int _size) { value = nullptr; SetSize(_size, _size); } 
    Array(int _size, int _maxsize) { value = nullptr;  SetSize(_size, _maxsize); }
    Array(const Array<T> & other);
    ~Array() { delete [] value; }

	void SetSize(int _size, int _maxs);

    void Add(const T &x);                      // push_back
    void Insert(int i, const T &x);
    void Remove(int i);
                                               //     Array<int> a(10);
    T & operator[](int i);                     //     a[i]=1;     x = a[i];
    const T & operator[](int i) const;         //     const Array<int> & b = a;
                                               //     x = b[i];  // b[i] = 1; 
    int Size()  const {return size;}

    bool TestInd(int i) const { return (0<=i && i<size) ? true : false; }
};


// implementations


template <class T>
Array<T>::Array(const Array<T> & other)
{
   value = nullptr;  
   SetSize(other.size, other.maxsize);
   for (int i=0; i<size; i++) { 
      value[i] = other.value[i];
   }
}


template <class T>
void Array<T>::SetSize(int _size, int _maxsize)
{
    char msg[64];
    if (_size > _maxsize) {
       sprintf(msg, "Array::SetSize: size(%d) > maxsize(%d)\n", _size, _maxsize);
       throw new MyException("Array::SetSize: size > maxsize", EC_RANGE);
    } 
    try {
       delete [] value;
       maxsize = _maxsize;
       value = new T[maxsize];
       size = _size;
    } catch (std::bad_alloc e) {
       sprintf(msg, "Array::SetSize: cannot allocate size %d\n", maxsize*sizeof(T));
       throw new MyException(msg, EC_MEMORY);
    }
}                      


template <class T>
void Array<T>::Add(const T &x)
{
 try {
   if (size == maxsize) {
       maxsize = 2*size; //     !!!
       T * new_value = new T[maxsize];
       for (int i=0; i<size; i++) { new_value[i] = value[i]; }
       delete [] value;
       value = new_value;
   }
   value[size++] = x;
   } catch (std::bad_alloc e) {
       char msg[64];
       sprintf(msg, "Array::Add: cannot allocate size %d\n", maxsize*sizeof(T));
       throw new MyException(msg, EC_MEMORY);
 }
}

template <class T>
void Array<T>::Remove(int i)
{
#ifdef _DEBUG_IND_CHECK
    if (!TestInd(i)) {
        char msg[64];
        printf("err\n");
        sprintf(msg,"Array::Remove: bad index i=%d size=%d\n", i, size);
        throw new MyException(msg, EC_RANGE);
    }
#endif
    for ( ; i<size - 1; i++) value[i] = value[i+1];  //    
    --size;
}                                                    // memcpy   memmove

template <class T>
void Array<T>::Insert(int i, const T & x)
{
//        
//  try    
//      maxsize   
//       ,     
//          c   0  i-1
//      i  size-1    
//  value[i] = x;
}

template <class T>
T & Array<T>::operator[](int i)
{
//    
   return value[i];
}

template <class T>
const T & Array<T>::operator[](int i) const
{
//    
   return value[i];
}



